@@ -16,7 +16,7 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <Python.h>
+#include "lib/replace/system/python.h"
 #include "python/py3compat.h"
 #include "includes.h"
 #include "python/modules.h"
@@ -41,6 +41,11 @@ static PyObject *py_creds_new(PyTypeObject *type, PyObject *args, PyObject *kwar
 	return pytalloc_steal(type, cli_credentials_init(NULL));
 }
 
+static PyObject *PyCredentials_from_cli_credentials(struct cli_credentials *creds)
+{
+	return pytalloc_reference(&PyCredentials, creds);
+}
+
 static PyObject *py_creds_get_username(PyObject *self, PyObject *unused)
 {
 	struct cli_credentials *creds = PyCredentials_AsCliCredentials(self);
@@ -389,7 +394,7 @@ static PyObject *py_creds_set_bind_dn(PyObject *self, PyObject *args)
 		PyErr_Format(PyExc_TypeError, "Credentials expected");
 		return NULL;
 	}
-	if (!PyArg_ParseTuple(args, "s", &newval))
+	if (!PyArg_ParseTuple(args, "z", &newval))
 		return NULL;
 
 	return PyBool_FromLong(cli_credentials_set_bind_dn(creds, newval));
@@ -546,6 +551,37 @@ static PyObject *py_creds_get_nt_hash(PyObject *self, PyObject *unused)
 	return ret;
 }
 
+static PyObject *py_creds_set_nt_hash(PyObject *self, PyObject *args)
+{
+	PyObject *py_cp = Py_None;
+	const struct samr_Password *pwd = NULL;
+	enum credentials_obtained obt = CRED_SPECIFIED;
+	int _obt = obt;
+	struct cli_credentials *creds = PyCredentials_AsCliCredentials(self);
+	if (creds == NULL) {
+		PyErr_Format(PyExc_TypeError, "Credentials expected");
+		return NULL;
+	}
+
+	if (!PyArg_ParseTuple(args, "O|i", &py_cp, &_obt)) {
+		return NULL;
+	}
+	obt = _obt;
+
+	if (!py_check_dcerpc_type(py_cp, "samba.dcerpc.samr", "Password")) {
+		/* py_check_dcerpc_type sets TypeError */
+		return NULL;
+	}
+
+	pwd = pytalloc_get_type(py_cp, struct samr_Password);
+	if (pwd == NULL) {
+		/* pytalloc_get_type sets TypeError */
+		return NULL;
+	}
+
+	return PyBool_FromLong(cli_credentials_set_nt_hash(creds, pwd, obt));
+}
+
 static PyObject *py_creds_get_kerberos_state(PyObject *self, PyObject *unused)
 {
 	int state;
@@ -1033,10 +1069,20 @@ static PyObject *py_creds_encrypt_samr_password(PyObject *self,
 		return NULL;
 	}
 
+	if (creds->netlogon_creds == NULL) {
+		PyErr_Format(PyExc_ValueError, "NetLogon credentials not set");
+		return NULL;
+	}
+
 	if (!PyArg_ParseTuple(args, "O", &py_cp)) {
 		return NULL;
 	}
 
+	if (!py_check_dcerpc_type(py_cp, "samba.dcerpc.samr", "Password")) {
+		/* py_check_dcerpc_type sets TypeError */
+		return NULL;
+	}
+
 	pwd = pytalloc_get_type(py_cp, struct samr_Password);
 	if (pwd == NULL) {
 		/* pytalloc_get_type sets TypeError */
@@ -1188,6 +1234,74 @@ static PyObject *py_creds_set_smb_encryption(PyObject *self, PyObject *args)
 	Py_RETURN_NONE;
 }
 
+static PyObject *py_creds_get_krb5_fast_armor_credentials(PyObject *self, PyObject *unused)
+{
+	struct cli_credentials *creds = NULL;
+	struct cli_credentials *fast_creds = NULL;
+
+	creds = PyCredentials_AsCliCredentials(self);
+	if (creds == NULL) {
+		PyErr_Format(PyExc_TypeError, "Credentials expected");
+		return NULL;
+	}
+
+	fast_creds = cli_credentials_get_krb5_fast_armor_credentials(creds);
+	if (fast_creds == NULL) {
+		Py_RETURN_NONE;
+	}
+
+	return PyCredentials_from_cli_credentials(fast_creds);
+}
+
+static PyObject *py_creds_set_krb5_fast_armor_credentials(PyObject *self, PyObject *args)
+{
+	struct cli_credentials *creds = NULL;
+	PyObject *pyfast_creds;
+	struct cli_credentials *fast_creds = NULL;
+	int fast_armor_required = 0;
+	NTSTATUS status;
+
+	creds = PyCredentials_AsCliCredentials(self);
+	if (creds == NULL) {
+		PyErr_Format(PyExc_TypeError, "Credentials expected");
+		return NULL;
+	}
+	if (!PyArg_ParseTuple(args, "Op", &pyfast_creds, &fast_armor_required)) {
+		return NULL;
+	}
+	if (pyfast_creds == Py_None) {
+		fast_creds = NULL;
+	} else {
+		fast_creds = PyCredentials_AsCliCredentials(pyfast_creds);
+		if (fast_creds == NULL) {
+			PyErr_Format(PyExc_TypeError, "Credentials expected");
+			return NULL;
+		}
+	}
+
+	status = cli_credentials_set_krb5_fast_armor_credentials(creds,
+								 fast_creds,
+								 fast_armor_required);
+
+	PyErr_NTSTATUS_IS_ERR_RAISE(status);
+	Py_RETURN_NONE;
+}
+
+static PyObject *py_creds_get_krb5_require_fast_armor(PyObject *self, PyObject *unused)
+{
+	bool krb5_fast_armor_required;
+	struct cli_credentials *creds = NULL;
+
+	creds = PyCredentials_AsCliCredentials(self);
+	if (creds == NULL) {
+		PyErr_Format(PyExc_TypeError, "Credentials expected");
+		return NULL;
+	}
+
+	krb5_fast_armor_required = cli_credentials_get_krb5_require_fast_armor(creds);
+	return PyBool_FromLong(krb5_fast_armor_required);
+}
+
 static PyMethodDef py_creds_methods[] = {
 	{
 		.ml_name  = "get_username",
@@ -1384,6 +1498,13 @@ static PyMethodDef py_creds_methods[] = {
 		.ml_meth  = py_creds_get_nt_hash,
 		.ml_flags = METH_NOARGS,
 	},
+	{
+		.ml_name  = "set_nt_hash",
+		.ml_meth  = py_creds_set_nt_hash,
+		.ml_flags = METH_VARARGS,
+		.ml_doc = "S.set_net_sh(samr_Password[, credentials.SPECIFIED]) -> bool\n"
+			"Change NT hash.",
+	},
 	{
 		.ml_name  = "get_kerberos_state",
 		.ml_meth  = py_creds_get_kerberos_state,
@@ -1520,6 +1641,27 @@ static PyMethodDef py_creds_methods[] = {
 		.ml_meth  = py_creds_set_smb_encryption,
 		.ml_flags = METH_VARARGS,
 	},
+	{
+		.ml_name  = "get_krb5_fast_armor_credentials",
+		.ml_meth  = py_creds_get_krb5_fast_armor_credentials,
+		.ml_flags = METH_NOARGS,
+		.ml_doc   = "S.get_krb5_fast_armor_credentials() -> Credentials\n"
+			    "Get the Kerberos FAST credentials set on this credentials object"
+	},
+	{
+		.ml_name  = "set_krb5_fast_armor_credentials",
+		.ml_meth  = py_creds_set_krb5_fast_armor_credentials,
+		.ml_flags = METH_VARARGS,
+		.ml_doc   = "S.set_krb5_fast_armor_credentials(credentials, required) -> None\n"
+			    "Set Kerberos FAST credentials for this credentials object, and if FAST armoring must be used."
+	},
+	{
+		.ml_name  = "get_krb5_require_fast_armor",
+		.ml_meth  = py_creds_get_krb5_require_fast_armor,
+		.ml_flags = METH_NOARGS,
+		.ml_doc   = "S.get_krb5_fast_armor() -> bool\n"
+			    "Indicate if Kerberos FAST armor is required"
+	},
 	{ .ml_name = NULL }
 };
 
@@ -1551,7 +1693,7 @@ static PyObject *py_ccache_name(PyObject *self, PyObject *unused)
 				    ccc->ccache, &name);
 	if (ret == 0) {
 		py_name = PyString_FromStringOrNULL(name);
-		SAFE_FREE(name);
+		krb5_free_string(ccc->smb_krb5_context->krb5_context, name);
 	} else {
 		PyErr_SetString(PyExc_RuntimeError,
 				"Failed to get ccache name");
