@@ -1,4 +1,4 @@
-/* 
+/*
    ctdb ltdb code - server side
 
    Copyright (C) Andrew Tridgell  2007
@@ -7,12 +7,12 @@
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
@@ -51,7 +51,7 @@
  * This is the server-variant of the ctdb_ltdb_store function.
  * It contains logic to determine whether a record should be
  * stored or deleted. It also sends SCHEDULE_FOR_DELETION
- * controls to the local ctdb daemon if apporpriate.
+ * controls to the local ctdb daemon if appropriate.
  */
 static int ctdb_ltdb_store_server(struct ctdb_db_context *ctdb_db,
 				  TDB_DATA key,
@@ -287,7 +287,7 @@ static void lock_fetch_callback(void *p, bool locked)
       -1:    means that it failed to get the lock, and won't retry
       -2:    means that it failed to get the lock immediately, but will retry
  */
-int ctdb_ltdb_lock_requeue(struct ctdb_db_context *ctdb_db, 
+int ctdb_ltdb_lock_requeue(struct ctdb_db_context *ctdb_db,
 			   TDB_DATA key, struct ctdb_req_header *hdr,
 			   void (*recv_pkt)(void *, struct ctdb_req_header *),
 			   void *recv_context, bool ignore_generation)
@@ -296,7 +296,7 @@ int ctdb_ltdb_lock_requeue(struct ctdb_db_context *ctdb_db,
 	struct tdb_context *tdb = ctdb_db->ltdb->tdb;
 	struct lock_request *lreq;
 	struct lock_fetch_state *state;
-	
+
 	ret = tdb_chainlock_nonblock(tdb, key);
 
 	if (ret != 0 &&
@@ -341,17 +341,17 @@ int ctdb_ltdb_lock_requeue(struct ctdb_db_context *ctdb_db,
 }
 
 /*
-  a varient of ctdb_ltdb_lock_requeue that also fetches the record
+  a variant of ctdb_ltdb_lock_requeue that also fetches the record
  */
-int ctdb_ltdb_lock_fetch_requeue(struct ctdb_db_context *ctdb_db, 
-				 TDB_DATA key, struct ctdb_ltdb_header *header, 
+int ctdb_ltdb_lock_fetch_requeue(struct ctdb_db_context *ctdb_db,
+				 TDB_DATA key, struct ctdb_ltdb_header *header,
 				 struct ctdb_req_header *hdr, TDB_DATA *data,
 				 void (*recv_pkt)(void *, struct ctdb_req_header *),
 				 void *recv_context, bool ignore_generation)
 {
 	int ret;
 
-	ret = ctdb_ltdb_lock_requeue(ctdb_db, key, hdr, recv_pkt, 
+	ret = ctdb_ltdb_lock_requeue(ctdb_db, key, hdr, recv_pkt,
 				     recv_context, ignore_generation);
 	if (ret != 0) {
 		return ret;
@@ -461,8 +461,8 @@ int ctdb_update_persistent_health(struct ctdb_context *ctdb,
 		}
 	} else if (old_reason && num_healthy_nodes == 0) {
 		/*
-		 * If the reason indicates ok, but there where no healthy nodes
-		 * available, that it means, we have not recovered valid content
+		 * If the reason indicates ok, but there were no healthy nodes
+		 * available, it means that we have not recovered valid content
 		 * of the db. So if there's an old reason, prefix it with
 		 * "NO-HEALTHY-NODES - "
 		 */
@@ -718,8 +718,8 @@ int ctdb_set_db_readonly(struct ctdb_context *ctdb, struct ctdb_db_context *ctdb
 		DEBUG(DEBUG_CRIT,("Failed to asprintf the tracking database\n"));
 		return -1;
 	}
-	ctdb_db->rottdb = tdb_open(ropath, 
-			      ctdb->tunable.database_hash_size, 
+	ctdb_db->rottdb = tdb_open(ropath,
+			      ctdb->tunable.database_hash_size,
 			      TDB_NOLOCK|TDB_CLEAR_IF_FIRST|TDB_NOSYNC,
 			      O_CREAT|O_RDWR, 0600);
 	if (ctdb_db->rottdb == NULL) {
@@ -770,6 +770,11 @@ static int ctdb_local_attach(struct ctdb_context *ctdb, const char *db_name,
 			CTDB_NO_MEMORY(ctdb, ctdb_db->delete_queue);
 		}
 
+		ctdb_db->fetch_queue = trbt_create(ctdb_db, 0);
+		if (ctdb_db->fetch_queue == NULL) {
+			CTDB_NO_MEMORY(ctdb, ctdb_db->fetch_queue);
+		}
+
 		ctdb_db->ctdb_ltdb_store_fn = ctdb_ltdb_store_server;
 	}
 
@@ -837,8 +842,8 @@ static int ctdb_local_attach(struct ctdb_context *ctdb, const char *db_name,
 
 again:
 	ctdb_db->ltdb = tdb_wrap_open(ctdb_db, ctdb_db->db_path,
-				      ctdb->tunable.database_hash_size, 
-				      tdb_flags, 
+				      ctdb->tunable.database_hash_size,
+				      tdb_flags,
 				      O_CREAT|O_RDWR, mode);
 	if (ctdb_db->ltdb == NULL) {
 		struct stat st;
@@ -940,7 +945,7 @@ again:
 	tdb_add_flags(ctdb_db->ltdb->tdb, tdb_flags);
 
 
-	/* set up a rb tree we can use to track which records we have a 
+	/* set up a rb tree we can use to track which records we have a
 	   fetch-lock in-flight for so we can defer any additional calls
 	   for the same record.
 	 */
@@ -964,7 +969,7 @@ again:
 	/* setting this can help some high churn databases */
 	tdb_set_max_dead(ctdb_db->ltdb->tdb, ctdb->tunable.database_max_dead);
 
-	/* 
+	/*
 	   all databases support the "null" function. we need this in
 	   order to do forced migration of records
 	*/
@@ -975,7 +980,7 @@ again:
 		return -1;
 	}
 
-	/* 
+	/*
 	   all databases support the "fetch" function. we need this
 	   for efficient Samba3 ctdb fetch
 	*/
@@ -986,7 +991,7 @@ again:
 		return -1;
 	}
 
-	/* 
+	/*
 	   all databases support the "fetch_with_header" function. we need this
 	   for efficient readonly record fetches
 	*/
@@ -1075,7 +1080,7 @@ int ctdb_process_deferred_attach(struct ctdb_context *ctdb)
 {
 	struct ctdb_deferred_attach_context *da_ctx;
 
-	/* call it from the main event loop as soon as the current event 
+	/* call it from the main event loop as soon as the current event
 	   finishes.
 	 */
 	while ((da_ctx = ctdb->deferred_attach) != NULL) {
@@ -1130,9 +1135,16 @@ int32_t ctdb_control_db_attach(struct ctdb_context *ctdb,
 			return -1;
 		}
 
-		if (ctdb->recovery_mode == CTDB_RECOVERY_ACTIVE &&
-		    client->pid != ctdb->recoverd_pid &&
-		    ctdb->runstate < CTDB_RUNSTATE_RUNNING) {
+		if ((c->flags & CTDB_CTRL_FLAG_ATTACH_RECOVERY) &&
+		    ctdb->recovery_mode != CTDB_RECOVERY_ACTIVE) {
+			DBG_ERR("Attach from recovery refused because "
+				"recovery is not active\n");
+			return -1;
+		}
+
+		if (!(c->flags & CTDB_CTRL_FLAG_ATTACH_RECOVERY) &&
+		    (ctdb->recovery_mode == CTDB_RECOVERY_ACTIVE ||
+		     ctdb->runstate < CTDB_RUNSTATE_STARTUP)) {
 			struct ctdb_deferred_attach_context *da_ctx = talloc(client, struct ctdb_deferred_attach_context);
 
 			if (da_ctx == NULL) {
@@ -1261,17 +1273,10 @@ int32_t ctdb_control_db_detach(struct ctdb_context *ctdb, TDB_DATA indata,
 		return -1;
 	}
 
-	/* Detach database from recoverd */
-	if (ctdb_daemon_send_message(ctdb, ctdb->pnn,
-				     CTDB_SRVID_DETACH_DATABASE,
-				     indata) != 0) {
-		DEBUG(DEBUG_ERR, ("Unable to detach DB from recoverd\n"));
-		return -1;
-	}
-
 	/* Disable vacuuming and drop all vacuuming data */
 	talloc_free(ctdb_db->vacuum_handle);
 	talloc_free(ctdb_db->delete_queue);
+	talloc_free(ctdb_db->fetch_queue);
 
 	/* Terminate any deferred fetch */
 	talloc_free(ctdb_db->deferred_fetch);
@@ -1320,7 +1325,7 @@ static int ctdb_attach_persistent(struct ctdb_context *ctdb,
 		size_t len = strlen(de->d_name);
 		uint32_t node;
 		int invalid_name = 0;
-		
+
 		s = talloc_strdup(ctdb, de->d_name);
 		if (s == NULL) {
 			closedir(d);
@@ -1592,13 +1597,15 @@ int ctdb_set_db_sticky(struct ctdb_context *ctdb, struct ctdb_db_context *ctdb_d
 
 void ctdb_db_statistics_reset(struct ctdb_db_context *ctdb_db)
 {
-	struct ctdb_db_statistics_old *s = &ctdb_db->statistics;
-	int i;
+	unsigned int i;
 
 	for (i=0; i<MAX_HOT_KEYS; i++) {
-		if (s->hot_keys[i].key.dsize > 0) {
-			talloc_free(s->hot_keys[i].key.dptr);
+		if (ctdb_db->hot_keys[i].key.dsize > 0) {
+			TALLOC_FREE(ctdb_db->hot_keys[i].key.dptr);
+			ctdb_db->hot_keys[i].key.dsize = 0;
 		}
+		ctdb_db->hot_keys[i].count = 0;
+		ctdb_db->hot_keys[i].last_logged_count = 0;
 	}
 
 	ZERO_STRUCT(ctdb_db->statistics);
@@ -1610,8 +1617,8 @@ int32_t ctdb_control_get_db_statistics(struct ctdb_context *ctdb,
 {
 	struct ctdb_db_context *ctdb_db;
 	struct ctdb_db_statistics_old *stats;
-	int i;
-	int len;
+	unsigned int i;
+	size_t len;
 	char *ptr;
 
 	ctdb_db = find_ctdb_db(ctdb, db_id);
@@ -1622,7 +1629,13 @@ int32_t ctdb_control_get_db_statistics(struct ctdb_context *ctdb,
 
 	len = offsetof(struct ctdb_db_statistics_old, hot_keys_wire);
 	for (i = 0; i < MAX_HOT_KEYS; i++) {
-		len += ctdb_db->statistics.hot_keys[i].key.dsize;
+		struct ctdb_db_statistics_old *s = &ctdb_db->statistics;
+
+		s->hot_keys[i].key.dsize = ctdb_db->hot_keys[i].key.dsize;
+		s->hot_keys[i].key.dptr = ctdb_db->hot_keys[i].key.dptr;
+		s->hot_keys[i].count = ctdb_db->hot_keys[i].count;
+
+		len += s->hot_keys[i].key.dsize;
 	}
 
 	stats = talloc_size(outdata, len);
