@@ -28,6 +28,7 @@
 #include <tevent.h>
 #include "lib/async_req/async_sock.h"
 #include "lib/util/iov_buf.h"
+#include "lib/util/util_net.h"
 
 /* Note: lib/util/ is currently GPL */
 #include "lib/util/tevent_unix.h"
@@ -150,7 +151,8 @@ struct tevent_req *async_connect_send(
 	 * use TEVENT_FD_READ in addition until we have
 	 * TEVENT_FD_ERROR.
 	 */
-	state->fde = tevent_add_fd(ev, state, fd, TEVENT_FD_READ|TEVENT_FD_WRITE,
+	state->fde = tevent_add_fd(ev, state, fd,
+				   TEVENT_FD_ERROR|TEVENT_FD_WRITE,
 				   async_connect_connected, req);
 	if (state->fde == NULL) {
 		tevent_req_error(req, ENOMEM);
@@ -276,8 +278,10 @@ struct tevent_req *writev_send(TALLOC_CTX *mem_ctx, struct tevent_context *ev,
 	if (tevent_req_nomem(state->iov, req)) {
 		return tevent_req_post(req, ev);
 	}
-	state->flags = TEVENT_FD_WRITE|TEVENT_FD_READ;
-	state->err_on_readability = err_on_readability;
+	state->flags = TEVENT_FD_WRITE | TEVENT_FD_ERROR;
+	if (err_on_readability) {
+		state->flags |= TEVENT_FD_READ;
+	}
 
 	tevent_req_set_cleanup_fn(req, writev_cleanup);
 	tevent_req_set_cancel_fn(req, writev_cancel);
@@ -400,36 +404,21 @@ static void writev_handler(struct tevent_context *ev, struct tevent_fd *fde,
 	struct writev_state *state =
 		tevent_req_data(req, struct writev_state);
 
-	if ((state->flags & TEVENT_FD_READ) && (flags & TEVENT_FD_READ)) {
-		int ret, value;
+	if (flags & TEVENT_FD_ERROR) {
+		/*
+		 * There's an error, for legacy reasons
+		 * we just use EPIPE instead of a more
+		 * detailed error using
+		 * samba_socket_poll_or_sock_error().
+		 */
+		tevent_req_error(req, EPIPE);
+		return;
+	}
 
-		if (state->err_on_readability) {
-			/* Readable and the caller wants an error on read. */
-			tevent_req_error(req, EPIPE);
-			return;
-		}
-
-		/* Might be an error. Check if there are bytes to read */
-		ret = ioctl(state->fd, FIONREAD, &value);
-		/* FIXME - should we also check
-		   for ret == 0 and value == 0 here ? */
-		if (ret == -1) {
-			/* There's an error. */
-			tevent_req_error(req, EPIPE);
-			return;
-		}
-		/* A request for TEVENT_FD_READ will succeed from now and
-		   forevermore until the bytes are read so if there was
-		   an error we'll wait until we do read, then get it in
-		   the read callback function. Until then, remove TEVENT_FD_READ
-		   from the flags we're waiting for. */
-		state->flags &= ~TEVENT_FD_READ;
-		TEVENT_FD_NOT_READABLE(fde);
-
-		/* If not writable, we're done. */
-		if (!(flags & TEVENT_FD_WRITE)) {
-			return;
-		}
+	if (flags & TEVENT_FD_READ) {
+		/* Readable and the caller wants an error on read. */
+		tevent_req_error(req, EPIPE);
+		return;
 	}
 
 	writev_do(req, state);
@@ -651,8 +640,7 @@ static void wait_for_read_done(struct tevent_context *ev,
 		private_data, struct tevent_req);
 	struct wait_for_read_state *state =
 	    tevent_req_data(req, struct wait_for_read_state);
-	ssize_t nread;
-	char c;
+	int ret, available;
 
 	if ((flags & TEVENT_FD_READ) == 0) {
 		return;
@@ -663,26 +651,20 @@ static void wait_for_read_done(struct tevent_context *ev,
 		return;
 	}
 
-	nread = recv(state->fd, &c, 1, MSG_PEEK);
+	ret = ioctl(state->fd, FIONREAD, &available);
 
-	if (nread == 0) {
-		tevent_req_error(req, EPIPE);
-		return;
-	}
-
-	if ((nread == -1) && (errno == EINTR)) {
+	if ((ret == -1) && (errno == EINTR)) {
 		/* come back later */
 		return;
 	}
 
-	if ((nread == -1) && (errno == ENOTSOCK)) {
-		/* Ignore this specific error on pipes */
-		tevent_req_done(req);
+	if (ret == -1) {
+		tevent_req_error(req, errno);
 		return;
 	}
 
-	if (nread == -1) {
-		tevent_req_error(req, errno);
+	if (available == 0) {
+		tevent_req_error(req, EPIPE);
 		return;
 	}
 
@@ -704,8 +686,7 @@ bool wait_for_read_recv(struct tevent_req *req, int *perr)
 struct accept_state {
 	struct tevent_fd *fde;
 	int listen_sock;
-	socklen_t addrlen;
-	struct sockaddr_storage addr;
+	struct samba_sockaddr addr;
 	int sock;
 };
 
@@ -747,10 +728,12 @@ static void accept_handler(struct tevent_context *ev, struct tevent_fd *fde,
 		tevent_req_error(req, EIO);
 		return;
 	}
-	state->addrlen = sizeof(state->addr);
 
-	ret = accept(state->listen_sock, (struct sockaddr *)&state->addr,
-		     &state->addrlen);
+	state->addr.sa_socklen = sizeof(state->addr.u);
+
+	ret = accept(state->listen_sock,
+		     &state->addr.u.sa,
+		     &state->addr.sa_socklen);
 	if ((ret == -1) && (errno == EINTR)) {
 		/* retry */
 		return;
@@ -764,23 +747,28 @@ static void accept_handler(struct tevent_context *ev, struct tevent_fd *fde,
 	tevent_req_done(req);
 }
 
-int accept_recv(struct tevent_req *req, struct sockaddr_storage *paddr,
-		socklen_t *paddrlen, int *perr)
+int accept_recv(struct tevent_req *req,
+		int *listen_sock,
+		struct samba_sockaddr *paddr,
+		int *perr)
 {
 	struct accept_state *state = tevent_req_data(req, struct accept_state);
+	int sock = state->sock;
 	int err;
 
 	if (tevent_req_is_unix_error(req, &err)) {
 		if (perr != NULL) {
 			*perr = err;
 		}
+		tevent_req_received(req);
 		return -1;
 	}
+	if (listen_sock != NULL) {
+		*listen_sock = state->listen_sock;
+	}
 	if (paddr != NULL) {
-		memcpy(paddr, &state->addr, state->addrlen);
+		*paddr = state->addr;
 	}
-	if (paddrlen != NULL) {
-		*paddrlen = state->addrlen;
-	}
-	return state->sock;
+	tevent_req_received(req);
+	return sock;
 }
