@@ -1,4 +1,4 @@
-/* 
+/*
    ldb database library - ldif handlers for Samba
 
    Copyright (C) Andrew Tridgell 2005
@@ -7,7 +7,7 @@
      ** NOTE! The following LGPL license applies to the ldb
      ** library. This does NOT imply that all of Samba is released
      ** under the LGPL
-   
+
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
@@ -59,7 +59,7 @@ static int ldif_write_NDR(struct ldb_context *ldb, void *mem_ctx,
 		return ldb_handler_copy(ldb, mem_ctx, in, out);
 	}
 	p = talloc_size(mem_ctx, struct_size);
-	err = ndr_pull_struct_blob(in, mem_ctx, 
+	err = ndr_pull_struct_blob(in, mem_ctx,
 				   p, pull_fn);
 	if (err != NDR_ERR_SUCCESS) {
 		/* fail in not in mask_error mode */
@@ -74,7 +74,7 @@ static int ldif_write_NDR(struct ldb_context *ldb, void *mem_ctx,
 	out->data = (uint8_t *)ndr_print_struct_string(mem_ctx, print_fn, "NDR", p);
 	talloc_free(p);
 	if (out->data == NULL) {
-		return ldb_handler_copy(ldb, mem_ctx, in, out);		
+		return ldb_handler_copy(ldb, mem_ctx, in, out);
 	}
 	out->length = strlen((char *)out->data);
 	return 0;
@@ -95,18 +95,18 @@ static int ldif_read_objectSid(struct ldb_context *ldb, void *mem_ctx,
 		char p[in->length+1];
 		memcpy(p, in->data, in->length);
 		p[in->length] = '\0';
-		
+
 		ret = dom_sid_parse(p, &sid);
 		if (ret == false) {
 			return -1;
 		}
-		
+
 		*out = data_blob_talloc(mem_ctx, NULL,
 					ndr_size_dom_sid(&sid, 0));
 		if (out->data == NULL) {
 			return -1;
 		}
-	
+
 		ndr_err = ndr_push_struct_into_fixed_blob(out, &sid,
 				(ndr_push_flags_fn_t)ndr_push_dom_sid);
 		if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
@@ -144,42 +144,53 @@ bool ldif_comparision_objectSid_isString(const struct ldb_val *v)
 	}
 
 	if (strncmp("S-", (const char *)v->data, 2) != 0) return false;
-	
+
 	return true;
 }
 
 /*
   compare two objectSids
+
+  If the SIDs seem to be strings, they are converted to binary form.
 */
 static int ldif_comparison_objectSid(struct ldb_context *ldb, void *mem_ctx,
 				    const struct ldb_val *v1, const struct ldb_val *v2)
 {
-	if (ldif_comparision_objectSid_isString(v1) && ldif_comparision_objectSid_isString(v2)) {
-		return ldb_comparison_binary(ldb, mem_ctx, v1, v2);
-	} else if (ldif_comparision_objectSid_isString(v1)
-		   && !ldif_comparision_objectSid_isString(v2)) {
-		struct ldb_val v;
-		int ret;
-		if (ldif_read_objectSid(ldb, mem_ctx, v1, &v) != 0) {
-			/* Perhaps not a string after all */
-			return ldb_comparison_binary(ldb, mem_ctx, v1, v2);
+	bool v1_is_string = ldif_comparision_objectSid_isString(v1);
+	bool v2_is_string = ldif_comparision_objectSid_isString(v2);
+	struct ldb_val parsed_1 = {};
+	struct ldb_val parsed_2 = {};
+	int ret;
+	/*
+	 * If the ldb_vals look like SID strings (i.e. start with "S-"
+	 * or "s-"), we try to parse them as such. If that fails, we
+	 * assume they are binary SIDs, even though that's not really
+	 * possible -- the first two bytes of a struct dom_sid are the
+	 * version (1), and the number of sub-auths (<= 15), neither
+	 * of which are close to 'S' or '-'.
+	 */
+	if (v1_is_string) {
+		int r = ldif_read_objectSid(ldb, mem_ctx, v1, &parsed_1);
+		if (r == 0) {
+			v1 = &parsed_1;
 		}
-		ret = ldb_comparison_binary(ldb, mem_ctx, &v, v2);
-		talloc_free(v.data);
-		return ret;
-	} else if (!ldif_comparision_objectSid_isString(v1)
-		   && ldif_comparision_objectSid_isString(v2)) {
-		struct ldb_val v;
-		int ret;
-		if (ldif_read_objectSid(ldb, mem_ctx, v2, &v) != 0) {
-			/* Perhaps not a string after all */
-			return ldb_comparison_binary(ldb, mem_ctx, v1, v2);
-		}
-		ret = ldb_comparison_binary(ldb, mem_ctx, v1, &v);
-		talloc_free(v.data);
-		return ret;
 	}
-	return ldb_comparison_binary(ldb, mem_ctx, v1, v2);
+	if (v2_is_string) {
+		int r = ldif_read_objectSid(ldb, mem_ctx, v2, &parsed_2);
+		if (r == 0) {
+			v2 = &parsed_2;
+		}
+	}
+
+	ret = ldb_comparison_binary(ldb, mem_ctx, v1, v2);
+
+	if (v1_is_string) {
+		TALLOC_FREE(parsed_1.data);
+	}
+	if (v2_is_string) {
+		TALLOC_FREE(parsed_2.data);
+	}
+	return ret;
 }
 
 /*
@@ -208,7 +219,7 @@ static int extended_dn_read_SID(struct ldb_context *ldb, void *mem_ctx,
 			return 0;
 		}
 	}
-	
+
 	/* Perhaps not a string after all */
 	*out = data_blob_talloc(mem_ctx, NULL, in->length/2+1);
 
@@ -290,13 +301,13 @@ static int extended_dn_read_GUID(struct ldb_context *ldb, void *mem_ctx,
 	if (in->length != 32) {
 		return -1;
 	}
-		
+
 	*out = data_blob_talloc(mem_ctx, NULL, in->length/2+1);
-	
+
 	if (!out->data) {
 		return -1;
 	}
-	
+
 	(*out).length = strhex_to_str((char *)out->data, out->length,
 				      (const char *)in->data, in->length);
 
@@ -369,6 +380,21 @@ static int ldif_read_ntSecurityDescriptor(struct ldb_context *ldb, void *mem_ctx
 	struct security_descriptor *sd;
 	enum ndr_err_code ndr_err;
 
+	if (in->length >= 2 && isupper(in->data[0]) && in->data[1] == ':') {
+		/*
+		 * If it starts with an upper case character followed by ':',
+		 * we know it's not NDR, but most likely SDDL...
+		 */
+		const struct dom_sid *sid = samdb_domain_sid(ldb);
+
+		sd = sddl_decode(mem_ctx, (const char *)in->data, sid);
+		if (sd == NULL) {
+			return -1;
+		}
+
+		goto decoded;
+	}
+
 	sd = talloc(mem_ctx, struct security_descriptor);
 	if (sd == NULL) {
 		return -1;
@@ -377,16 +403,11 @@ static int ldif_read_ntSecurityDescriptor(struct ldb_context *ldb, void *mem_ctx
 	ndr_err = ndr_pull_struct_blob(in, sd, sd,
 				       (ndr_pull_flags_fn_t)ndr_pull_security_descriptor);
 	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
-		/* If this does not parse, then it is probably SDDL, and we should try it that way */
-
-		const struct dom_sid *sid = samdb_domain_sid(ldb);
 		talloc_free(sd);
-		sd = sddl_decode(mem_ctx, (const char *)in->data, sid);
-		if (sd == NULL) {
-			return -1;
-		}
+		return -1;
 	}
 
+decoded:
 	ndr_err = ndr_push_struct_blob(out, mem_ctx, sd,
 				       (ndr_push_flags_fn_t)ndr_push_security_descriptor);
 	talloc_free(sd);
@@ -407,12 +428,12 @@ static int ldif_write_ntSecurityDescriptor(struct ldb_context *ldb, void *mem_ct
 	enum ndr_err_code ndr_err;
 
 	if (ldb_get_flags(ldb) & LDB_FLG_SHOW_BINARY) {
-		return ldif_write_NDR(ldb, mem_ctx, in, out, 
+		return ldif_write_NDR(ldb, mem_ctx, in, out,
 				      sizeof(struct security_descriptor),
 				      (ndr_pull_flags_fn_t)ndr_pull_security_descriptor,
 				      (ndr_print_fn_t)ndr_print_security_descriptor,
 				      true);
-				      
+
 	}
 
 	sd = talloc(mem_ctx, struct security_descriptor);
@@ -457,7 +478,7 @@ static int ldif_write_sddlSecurityDescriptor(struct ldb_context *ldb, void *mem_
 	return ldb_handler_copy(ldb, mem_ctx, in, out);
 }
 
-/* 
+/*
    canonicalise an objectCategory.  We use the long form as the canonical form:
    'person' becomes cn=Person,cn=Schema,cn=Configuration,<basedn>
 
@@ -588,7 +609,7 @@ static int ldif_read_prefixMap(struct ldb_context *ldb, void *mem_ctx,
 
 	/* If this does not parse, then it is probably the text version, and we should try it that way */
 	blob->version = PREFIX_MAP_VERSION_DSDB;
-	
+
 	string = talloc_strndup(mem_ctx, (const char *)in->data, in->length);
 	if (string == NULL) {
 		talloc_free(blob);
@@ -612,9 +633,9 @@ static int ldif_read_prefixMap(struct ldb_context *ldb, void *mem_ctx,
 		if (line == p) {
 			break;
 		}
-		
-		blob->ctr.dsdb.mappings = talloc_realloc(blob, 
-							 blob->ctr.dsdb.mappings, 
+
+		blob->ctr.dsdb.mappings = talloc_realloc(blob,
+							 blob->ctr.dsdb.mappings,
 							 struct drsuapi_DsReplicaOIDMapping,
 							 blob->ctr.dsdb.num_mappings+1);
 		if (!blob->ctr.dsdb.mappings) {
@@ -650,7 +671,7 @@ static int ldif_read_prefixMap(struct ldb_context *ldb, void *mem_ctx,
 		}
 	}
 
-	ndr_err = ndr_push_struct_blob(out, mem_ctx, 
+	ndr_err = ndr_push_struct_blob(out, mem_ctx,
 				       blob,
 				       (ndr_push_flags_fn_t)ndr_push_prefixMapBlob);
 	talloc_free(tmp_ctx);
@@ -694,7 +715,7 @@ static int ldif_write_prefixMap(struct ldb_context *ldb, void *mem_ctx,
 	if (blob == NULL) {
 		return -1;
 	}
-	ndr_err = ndr_pull_struct_blob_all(in, blob, 
+	ndr_err = ndr_pull_struct_blob_all(in, blob,
 					   blob,
 					   (ndr_pull_flags_fn_t)ndr_pull_prefixMapBlob);
 	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
@@ -713,23 +734,20 @@ static int ldif_write_prefixMap(struct ldb_context *ldb, void *mem_ctx,
 		char *partial_oid = NULL;
 
 		if (i > 0) {
-			string = talloc_asprintf_append(string, ";"); 
+			talloc_asprintf_addbuf(&string, ";");
 		}
 
 		oid_blob = data_blob_const(blob->ctr.dsdb.mappings[i].oid.binary_oid,
 					   blob->ctr.dsdb.mappings[i].oid.length);
 		if (!ber_read_partial_OID_String(blob, oid_blob, &partial_oid)) {
-			DEBUG(0, ("ber_read_partial_OID failed on prefixMap item with id: 0x%X",
+			DEBUG(0, ("ber_read_partial_OID failed on prefixMap item with id: 0x%X\n",
 				  blob->ctr.dsdb.mappings[i].id_prefix));
 			goto failed;
 		}
-		string = talloc_asprintf_append(string, "%u:%s", 
-						   blob->ctr.dsdb.mappings[i].id_prefix,
-						   partial_oid);
+		talloc_asprintf_addbuf(&string, "%u:%s",
+				       blob->ctr.dsdb.mappings[i].id_prefix,
+				       partial_oid);
 		talloc_free(discard_const(partial_oid));
-		if (string == NULL) {
-			goto failed;
-		}
 	}
 
 	talloc_free(blob);
@@ -750,7 +768,7 @@ static bool ldif_comparision_prefixMap_isString(const struct ldb_val *v)
 	if (IVAL(v->data, 0) == PREFIX_MAP_VERSION_DSDB) {
 		return false;
 	}
-	
+
 	return true;
 }
 
@@ -947,7 +965,7 @@ static int ldif_comparison_int64(struct ldb_context *ldb, void *mem_ctx,
 static int ldif_write_repsFromTo(struct ldb_context *ldb, void *mem_ctx,
 				 const struct ldb_val *in, struct ldb_val *out)
 {
-	return ldif_write_NDR(ldb, mem_ctx, in, out, 
+	return ldif_write_NDR(ldb, mem_ctx, in, out,
 			      sizeof(struct repsFromToBlob),
 			      (ndr_pull_flags_fn_t)ndr_pull_repsFromToBlob,
 			      (ndr_print_fn_t)ndr_print_repsFromToBlob,
@@ -960,7 +978,7 @@ static int ldif_write_repsFromTo(struct ldb_context *ldb, void *mem_ctx,
 static int ldif_write_replPropertyMetaData(struct ldb_context *ldb, void *mem_ctx,
 					   const struct ldb_val *in, struct ldb_val *out)
 {
-	return ldif_write_NDR(ldb, mem_ctx, in, out, 
+	return ldif_write_NDR(ldb, mem_ctx, in, out,
 			      sizeof(struct replPropertyMetaDataBlob),
 			      (ndr_pull_flags_fn_t)ndr_pull_replPropertyMetaDataBlob,
 			      (ndr_print_fn_t)ndr_print_replPropertyMetaDataBlob,
@@ -973,7 +991,7 @@ static int ldif_write_replPropertyMetaData(struct ldb_context *ldb, void *mem_ct
 static int ldif_write_replUpToDateVector(struct ldb_context *ldb, void *mem_ctx,
 					 const struct ldb_val *in, struct ldb_val *out)
 {
-	return ldif_write_NDR(ldb, mem_ctx, in, out, 
+	return ldif_write_NDR(ldb, mem_ctx, in, out,
 			      sizeof(struct replUpToDateVectorBlob),
 			      (ndr_pull_flags_fn_t)ndr_pull_replUpToDateVectorBlob,
 			      (ndr_print_fn_t)ndr_print_replUpToDateVectorBlob,
@@ -1141,22 +1159,41 @@ static int samba_ldb_dn_link_comparison(struct ldb_context *ldb, void *mem_ctx,
 	struct ldb_dn *dn1 = NULL, *dn2 = NULL;
 	int ret;
 
+	/*
+	 * In a sort context, Deleted DNs get shifted to the end.
+	 * They never match in an equality
+	 */
 	if (dsdb_dn_is_deleted_val(v1)) {
-		/* If the DN is deleted, then we can't search for it */
-		return -1;
-	}
-
-	if (dsdb_dn_is_deleted_val(v2)) {
-		/* If the DN is deleted, then we can't search for it */
+		if (! dsdb_dn_is_deleted_val(v2)) {
+			return 1;
+		}
+		/*
+		 * They are both deleted!
+		 *
+		 * The soundest thing to do at this point is carry on
+		 * and compare the DNs normally. This matches the
+		 * behaviour of samba_dn_extended_match() below.
+		 */
+	} else if (dsdb_dn_is_deleted_val(v2)) {
 		return -1;
 	}
 
 	dn1 = ldb_dn_from_ldb_val(mem_ctx, ldb, v1);
-	if ( ! ldb_dn_validate(dn1)) return -1;
-
 	dn2 = ldb_dn_from_ldb_val(mem_ctx, ldb, v2);
+
+	if ( ! ldb_dn_validate(dn1)) {
+		TALLOC_FREE(dn1);
+		if ( ! ldb_dn_validate(dn2)) {
+			TALLOC_FREE(dn2);
+			return 0;
+		}
+		TALLOC_FREE(dn2);
+		return 1;
+	}
+
 	if ( ! ldb_dn_validate(dn2)) {
-		talloc_free(dn1);
+		TALLOC_FREE(dn1);
+		TALLOC_FREE(dn2);
 		return -1;
 	}
 
@@ -1314,7 +1351,7 @@ static int samba_syntax_operator_fn(struct ldb_context *ldb, enum ldb_parse_op o
 
 /*
   compare two binary objects.  This is correct for sorting as the sort order is:
-  
+
   a
   aa
   b
@@ -1326,7 +1363,7 @@ static int samba_syntax_operator_fn(struct ldb_context *ldb, enum ldb_parse_op o
   b
   aa
   bb
-  
+
 */
 static int samba_ldb_comparison_binary(struct ldb_context *ldb, void *mem_ctx,
 				       const struct ldb_val *v1, const struct ldb_val *v2)
@@ -1749,7 +1786,7 @@ const struct ldb_schema_syntax *ldb_samba_syntax_by_name(struct ldb_context *ldb
 {
 	unsigned int j;
 	const struct ldb_schema_syntax *s = NULL;
-	
+
 	for (j=0; j < ARRAY_SIZE(samba_syntaxes); j++) {
 		if (strcmp(name, samba_syntaxes[j].name) == 0) {
 			s = &samba_syntaxes[j];
@@ -1770,7 +1807,7 @@ const struct ldb_schema_syntax *ldb_samba_syntax_by_lDAPDisplayName(struct ldb_c
 			break;
 		}
 	}
-	
+
 	return s;
 }
 
