@@ -6,7 +6,7 @@ set -u
 
 ACCOUNT_NAME="${SAMBA_CPS_ACCOUNT_NAME}"
 INVALIDPW="$1"
-NEWPW=`cat -`
+NEWPW=$(cat -)
 
 echo -n "${NEWPW}" | grep -q "^${INVALIDPW}\$" && {
 	echo "Found invalid password" >&1
