@@ -132,7 +132,7 @@
  *
  * The ad_xattr_header follows the FinderInfo data in the FinderInfo entry if
  * the length of the FinderInfo entry is larger then 32 bytes. It is then
- * preceeded with 2 bytes padding.
+ * preceded with 2 bytes padding.
  *
  * Cf: https://opensource.apple.com/source/xnu/xnu-4570.1.46/bsd/vfs/vfs_xattr.c
  */
@@ -328,7 +328,7 @@ static bool ad_entry_check_size(uint32_t eid,
 	}
 	if (ad_checks[eid].fixed_size) {
 		if (ad_checks[eid].expected_len != got_len) {
-			/* Wrong size fo fixed size entry. */
+			/* Wrong size for fixed size entry. */
 			return false;
 		}
 	} else {
@@ -967,14 +967,14 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 	 */
 
 	if (bufsize < (AD_HEADER_LEN + (AD_ENTRY_LEN * nentries))) {
-		DEBUG(1, ("bad size\n"));
+		DBG_NOTICE("Bad size\n");
 		return false;
 	}
 
 	ad->ad_magic = RIVAL(ad->ad_data, 0);
 	ad->ad_version = RIVAL(ad->ad_data, ADEDOFF_VERSION);
 	if ((ad->ad_magic != AD_MAGIC) || (ad->ad_version != AD_VERSION)) {
-		DEBUG(1, ("wrong magic or version\n"));
+		DBG_NOTICE("Wrong magic or version\n");
 		return false;
 	}
 
@@ -982,8 +982,7 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 
 	adentries = RSVAL(ad->ad_data, ADEDOFF_NENTRIES);
 	if (adentries != nentries) {
-		DEBUG(1, ("invalid number of entries: %zu\n",
-			  adentries));
+		DBG_NOTICE("Invalid number of entries: %zu\n", adentries);
 		return false;
 	}
 
@@ -995,7 +994,7 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 		len = RIVAL(ad->ad_data, AD_HEADER_LEN + (i * AD_ENTRY_LEN) + 8);
 
 		if (!eid || eid >= ADEID_MAX) {
-			DEBUG(1, ("bogus eid %d\n", eid));
+			DBG_NOTICE("Bogus eid %d\n", eid);
 			return false;
 		}
 
@@ -1005,16 +1004,22 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 		 * ensure the specified offset is within that bound
 		 */
 		if ((off > bufsize) && (eid != ADEID_RFORK)) {
-			DEBUG(1, ("bogus eid %d: off: %" PRIu32 ", len: %" PRIu32 "\n",
-				  eid, off, len));
+			DBG_NOTICE("Bogus eid %d: off: %" PRIu32
+				   ", len: %" PRIu32 "\n",
+				   eid,
+				   off,
+				   len);
 			return false;
 		}
 
 		ok = ad_entry_check_size(eid, bufsize, off, len);
 		if (!ok) {
-			DBG_ERR("bogus eid [%"PRIu32"] bufsize [%zu] "
-				"off [%"PRIu32"] len [%"PRIu32"]\n",
-				eid, bufsize, off, len);
+			DBG_NOTICE("Bogus eid [%" PRIu32 "] bufsize [%zu] "
+				   "off [%" PRIu32 "] len [%" PRIu32 "]\n",
+				   eid,
+				   bufsize,
+				   off,
+				   len);
 			return false;
 		}
 
@@ -1022,8 +1027,11 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 		 * That would be obviously broken
 		 */
 		if (off > filesize) {
-			DEBUG(1, ("bogus eid %d: off: %" PRIu32 ", len: %" PRIu32 "\n",
-				  eid, off, len));
+			DBG_NOTICE("Bogus eid %d: off: %" PRIu32
+				   ", len: %" PRIu32 "\n",
+				   eid,
+				   off,
+				   len);
 			return false;
 		}
 
@@ -1032,9 +1040,11 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 		 * filesize.
 		 */
 		if (off + len < off) {
-			DEBUG(1, ("offset wrap in eid %d: off: %" PRIu32
-				  ", len: %" PRIu32 "\n",
-				  eid, off, len));
+			DBG_NOTICE("offset wrap in eid %d: off: %" PRIu32
+				   ", len: %" PRIu32 "\n",
+				   eid,
+				   off,
+				   len);
 			return false;
 
 		}
@@ -1045,9 +1055,11 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 			 * out.
 			 */
 			if (eid != ADEID_RFORK) {
-				DEBUG(1, ("bogus eid %d: off: %" PRIu32
-					  ", len: %" PRIu32 "\n",
-					  eid, off, len));
+				DBG_NOTICE("Bogus eid %d: off: %" PRIu32
+					   ", len: %" PRIu32 "\n",
+					   eid,
+					   off,
+					   len);
 				return false;
 			}
 
@@ -1056,8 +1068,10 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 			 * the size to entryoffset - filesize.
 			 */
 			len = filesize - off;
-			DEBUG(1, ("Limiting ADEID_RFORK: off: %" PRIu32
-				  ", len: %" PRIu32 "\n", off, len));
+			DBG_NOTICE("Limiting ADEID_RFORK: off: %" PRIu32
+				   ", len: %" PRIu32 "\n",
+				   off,
+				   len);
 		}
 
 		ad->ad_eid[eid].ade_off = off;
@@ -2151,7 +2165,11 @@ static ssize_t ad_read_meta(vfs_handle_struct *handle,
 	/* Now parse entries */
 	ok = ad_unpack(ad, ADEID_NUM_XATTR, AD_DATASZ_XATTR);
 	if (!ok) {
-		DEBUG(2, ("invalid AppleDouble metadata xattr\n"));
+		DBG_WARNING(
+			"Invalid AppleDouble xattr metadata (%s) in file: %s - "
+			"Consider deleting the corrupted file.\n",
+			smb_fname->base_name,
+			ad->ad_fsp->fsp_name->base_name);
 		errno = EINVAL;
 		rc = -1;
 		goto exit;
@@ -2360,8 +2378,10 @@ static ssize_t ad_read_rsrc_adouble(vfs_handle_struct *handle,
 		       ADEID_NUM_DOT_UND,
 		       ad->ad_fsp->fsp_name->st.st_ex_size);
 	if (!ok) {
-		DBG_ERR("invalid AppleDouble resource %s\n",
-			smb_fname->base_name);
+		DBG_WARNING("Invalid AppleDouble resource (%s) in file: %s - "
+			    "Consider deleting the corrupted file.\n",
+			    smb_fname->base_name,
+			    ad->ad_fsp->fsp_name->base_name);
 		errno = EINVAL;
 		return -1;
 	}
@@ -2370,8 +2390,10 @@ static ssize_t ad_read_rsrc_adouble(vfs_handle_struct *handle,
 	    || (ad_getentrylen(ad, ADEID_FINDERI) < ADEDLEN_FINDERI)
 	    || (ad_getentryoff(ad, ADEID_RFORK) < ADEDOFF_RFORK_DOT_UND))
 	{
-		DBG_ERR("invalid AppleDouble resource %s\n",
-			smb_fname->base_name);
+		DBG_WARNING("Invalid AppleDouble resource (%s) in file: %s - "
+			    "Consider deleting the corrupted file.\n",
+			    smb_fname->base_name,
+			    ad->ad_fsp->fsp_name->base_name);
 		errno = EINVAL;
 		return -1;
 	}
@@ -2685,7 +2707,7 @@ int ad_fset(struct vfs_handle_struct *handle,
 					  ad_getentryoff(ad, ADEID_RFORK),
 					  0);
 		if (len != ad_getentryoff(ad, ADEID_RFORK)) {
-			DBG_ERR("short write on %s: %zd", fsp_str_dbg(fsp), len);
+			DBG_ERR("short write on %s: %zd\n", fsp_str_dbg(fsp), len);
 			return -1;
 		}
 		rc = 0;
@@ -2805,7 +2827,7 @@ ssize_t afpinfo_pack(const AfpInfo *ai, char *buf)
  * Buffer size must be at least AFP_INFO_SIZE
  * Returns allocated AfpInfo struct
  **/
-AfpInfo *afpinfo_unpack(TALLOC_CTX *ctx, const void *data)
+AfpInfo *afpinfo_unpack(TALLOC_CTX *ctx, const void *data, bool validate)
 {
 	AfpInfo *ai = talloc_zero(ctx, AfpInfo);
 	if (ai == NULL) {
@@ -2818,10 +2840,16 @@ AfpInfo *afpinfo_unpack(TALLOC_CTX *ctx, const void *data)
 	memcpy(ai->afpi_FinderInfo, (const char *)data + 16,
 	       sizeof(ai->afpi_FinderInfo));
 
-	if (ai->afpi_Signature != AFP_Signature
-	    || ai->afpi_Version != AFP_Version) {
-		DEBUG(1, ("Bad AfpInfo signature or version\n"));
-		TALLOC_FREE(ai);
+	if (validate) {
+		if (ai->afpi_Signature != AFP_Signature
+		    || ai->afpi_Version != AFP_Version)
+		{
+			DEBUG(1, ("Bad AfpInfo signature or version\n"));
+			TALLOC_FREE(ai);
+		}
+	} else {
+		ai->afpi_Signature = AFP_Signature;
+		ai->afpi_Version = AFP_Version;
 	}
 
 	return ai;
