@@ -445,7 +445,7 @@ static struct tevent_req *smbd_smb2_ioctl_send(TALLOC_CTX *mem_ctx,
 		   fsp ? fsp_str_dbg(fsp) : "<no handle>",
 		   fsp_fnum_dbg(fsp)));
 
-	smbreq = smbd_smb2_fake_smb_request(smb2req);
+	smbreq = smbd_smb2_fake_smb_request(smb2req, fsp);
 	if (tevent_req_nomem(smbreq, req)) {
 		return tevent_req_post(req, ev);
 	}
@@ -454,19 +454,14 @@ static struct tevent_req *smbd_smb2_ioctl_send(TALLOC_CTX *mem_ctx,
 	switch (in_ctl_code & IOCTL_DEV_TYPE_MASK) {
 	case FSCTL_DFS:
 		return smb2_ioctl_dfs(in_ctl_code, ev, req, state);
-		break;
 	case FSCTL_FILESYSTEM:
 		return smb2_ioctl_filesys(in_ctl_code, ev, req, state);
-		break;
 	case FSCTL_NAMED_PIPE:
 		return smb2_ioctl_named_pipe(in_ctl_code, ev, req, state);
-		break;
 	case FSCTL_NETWORK_FILESYSTEM:
 		return smb2_ioctl_network_fs(in_ctl_code, ev, req, state);
-		break;
 	case FSCTL_SMBTORTURE:
 		return smb2_ioctl_smbtorture(in_ctl_code, ev, req, state);
-		break;
 	default:
 		if (IS_IPC(smbreq->conn)) {
 			tevent_req_nterror(req, NT_STATUS_FS_DRIVER_REQUIRED);
@@ -475,11 +470,7 @@ static struct tevent_req *smbd_smb2_ioctl_send(TALLOC_CTX *mem_ctx,
 		}
 
 		return tevent_req_post(req, ev);
-		break;
 	}
-
-	tevent_req_nterror(req, NT_STATUS_INTERNAL_ERROR);
-	return tevent_req_post(req, ev);
 }
 
 static NTSTATUS smbd_smb2_ioctl_recv(struct tevent_req *req,
