@@ -1,20 +1,20 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
 
    interface functions for the sam database
 
    Copyright (C) Andrew Tridgell 2004
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
@@ -45,6 +45,7 @@ enum dsdb_password_checked {
 #include "librpc/gen_ndr/drsuapi.h"
 #include "librpc/gen_ndr/drsblobs.h"
 #include "dsdb/schema/schema.h"
+#include "auth/session.h"
 #include "dsdb/samdb/samdb_proto.h"
 #include "dsdb/common/dsdb_dn.h"
 #include "dsdb/common/util_links.h"
@@ -53,7 +54,7 @@ enum dsdb_password_checked {
 
 #define DSDB_CONTROL_CURRENT_PARTITION_OID "1.3.6.1.4.1.7165.4.3.2"
 struct dsdb_control_current_partition {
-	/* 
+	/*
 	 * this is the version of the dsdb_control_current_partition
 	 * version 0: initial implementation
 	 * version 1: got rid of backend and module fields
@@ -149,7 +150,7 @@ struct dsdb_control_password_change {
 
 /*
  * passed from the descriptor module in order to
- * store the recalucated nTSecurityDescriptor without
+ * store the recalculated nTSecurityDescriptor without
  * modifying the replPropertyMetaData.
  */
 #define DSDB_CONTROL_SEC_DESC_PROPAGATION_OID "1.3.6.1.4.1.7165.4.3.21"
@@ -267,7 +268,7 @@ struct dsdb_extended_replicated_object {
 #define DSDB_EXTENDED_SCHEMA_UPDATE_NOW_OID "1.3.6.1.4.1.7165.4.4.2"
 
 struct dsdb_extended_replicated_objects {
-	/* 
+	/*
 	 * this is the version of the dsdb_extended_replicated_objects
 	 * version 0: initial implementation
 	 */
@@ -312,7 +313,7 @@ struct dsdb_fsmo_extended_op {
 
 /*
  * passed from the descriptor module in order to
- * store the recalucated nTSecurityDescriptor without
+ * store the recalculated nTSecurityDescriptor without
  * modifying the replPropertyMetaData.
  */
 #define DSDB_EXTENDED_SEC_DESC_PROPAGATION_OID "1.3.6.1.4.1.7165.4.4.7"
