@@ -26,6 +26,7 @@
 #include "libcli/raw/request.h"
 #include "librpc/gen_ndr/nbt.h"
 #include "libcli/raw/interfaces.h"
+#include "libcli/smb/smb2_negotiate_context.h"
 
 struct smbcli_tree;  /* forward declare */
 struct smbcli_request;  /* forward declare */
@@ -103,6 +104,9 @@ struct smbcli_options {
 	enum smb_signing_setting signing;
 	uint32_t smb2_capabilities;
 	struct GUID client_guid;
+	uint64_t max_credits;
+	unsigned int only_negprot;
+	struct smb311_capabilities smb3_capabilities;
 };
 
 /* this is the context for the client transport layer */
@@ -169,7 +173,7 @@ struct smbcli_session {
 	   the user to control these for torture testing */
 	uint16_t flags2;
 
-	/* the spnego context if we use extented security */
+	/* the spnego context if we use extended security */
 	struct gensec_security *gensec;
 
 	struct smbcli_session_options {
@@ -241,7 +245,7 @@ struct smbcli_request {
 	struct smb_nttrans nttrans;
 
 	/* information on what to do with a reply when it is received
-	   asyncronously. If this is not setup when a reply is received then
+	   asynchronously. If this is not setup when a reply is received then
 	   the reply is discarded
 
 	   The private pointer is private to the caller of the client
@@ -271,7 +275,6 @@ struct smbcli_request *smb_raw_read_send(struct smbcli_tree *tree, union smb_rea
 NTSTATUS smb_raw_trans_recv(struct smbcli_request *req,
 			     TALLOC_CTX *mem_ctx,
 			     struct smb_trans2 *parms);
-size_t smb_raw_max_trans_data(struct smbcli_tree *tree, size_t param_size);
 struct smbcli_request *smb_raw_trans_send(struct smbcli_tree *tree, struct smb_trans2 *parms);
 NTSTATUS smbcli_request_destroy(struct smbcli_request *req);
 struct smbcli_request *smb_raw_write_send(struct smbcli_tree *tree, union smb_write *parms);
