@@ -2725,7 +2725,7 @@ static WERROR becomeDC_drsuapi_pull_partition_recv(struct libnet_BecomeDC_state
 		ctr6 = &r->out.ctr->ctr7.ctr.mszip6.ts->ctr6;
 	} else if (*r->out.level_out == 7 &&
 		   r->out.ctr->ctr7.level == 6 &&
-		   r->out.ctr->ctr7.type == DRSUAPI_COMPRESSION_TYPE_XPRESS &&
+		   r->out.ctr->ctr7.type == DRSUAPI_COMPRESSION_TYPE_WIN2K3_LZ77_DIRECT2 &&
 		   r->out.ctr->ctr7.ctr.xpress6.ts) {
 		ctr_level = 6;
 		ctr6 = &r->out.ctr->ctr7.ctr.xpress6.ts->ctr6;
