@@ -375,9 +375,9 @@ static const char dda1d01d_ldif[] = ""
 "uSNChanged: 3467\n"
 "showInAdvancedViewOnly: TRUE\n"
 "nTSecurityDescriptor: O:S-1-5-21-2106703258-1007804629-1260019310-512G:S-1-5-2\n"
-" 1-2106703258-1007804629-1260019310-512D:AI(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;S-\n"
-" 1-5-21-2106703258-1007804629-1260019310-512)(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;\n"
-" SY)(A;;RPLCLORC;;;AU)(OA;CIIOID;RP;4c164200-20c0-11d0-a768-00aa006e0529;4828c\n"
+" 1-2106703258-1007804629-1260019310-512D:AI(A;;CCDCLCSWRPWPDTLOCRSDRCWDWO;;;S-\n"
+" 1-5-21-2106703258-1007804629-1260019310-512)(A;;CCDCLCSWRPWPDTLOCRSDRCWDWO;;;\n"
+" SY)(A;;LCRPLORC;;;AU)(OA;CIIOID;RP;4c164200-20c0-11d0-a768-00aa006e0529;4828c\n"
 " c14-1437-45bc-9b07-ad6f015e5f28;RU)(OA;CIIOID;RP;4c164200-20c0-11d0-a768-00aa\n"
 " 006e0529;bf967aba-0de6-11d0-a285-00aa003049e2;RU)(OA;CIIOID;RP;5f202010-79a5-\n"
 " 11d0-9020-00c04fc2d4cf;4828cc14-1437-45bc-9b07-ad6f015e5f28;RU)(OA;CIIOID;RP;\n"
@@ -392,12 +392,12 @@ static const char dda1d01d_ldif[] = ""
 " 9e2;RU)(OA;CIIOID;RP;b7c69e6d-2cc7-11d2-854e-00a0c983f608;bf967a86-0de6-11d0-\n"
 " a285-00aa003049e2;ED)(OA;CIIOID;RP;b7c69e6d-2cc7-11d2-854e-00a0c983f608;bf967\n"
 " a9c-0de6-11d0-a285-00aa003049e2;ED)(OA;CIIOID;RP;b7c69e6d-2cc7-11d2-854e-00a0\n"
-" c983f608;bf967aba-0de6-11d0-a285-00aa003049e2;ED)(OA;CIIOID;RPLCLORC;;4828cc1\n"
-" 4-1437-45bc-9b07-ad6f015e5f28;RU)(OA;CIIOID;RPLCLORC;;bf967a9c-0de6-11d0-a285\n"
-" -00aa003049e2;RU)(OA;CIIOID;RPLCLORC;;bf967aba-0de6-11d0-a285-00aa003049e2;RU\n"
-" )(OA;CIID;RPWPCR;91e647de-d96f-4b70-9557-d63ff4f3ccd8;;PS)(A;CIID;RPWPCRCCDCL\n"
-" CLORCWOWDSDDTSW;;;S-1-5-21-2106703258-1007804629-1260019310-519)(A;CIID;LC;;;\n"
-" RU)(A;CIID;RPWPCRCCLCLORCWOWDSDSW;;;BA)S:AI(OU;CIIOIDSA;WP;f30e3bbe-9ff0-11d1\n"
+" c983f608;bf967aba-0de6-11d0-a285-00aa003049e2;ED)(OA;CIIOID;LCRPLORC;;4828cc1\n"
+" 4-1437-45bc-9b07-ad6f015e5f28;RU)(OA;CIIOID;LCRPLORC;;bf967a9c-0de6-11d0-a285\n"
+" -00aa003049e2;RU)(OA;CIIOID;LCRPLORC;;bf967aba-0de6-11d0-a285-00aa003049e2;RU\n"
+" )(OA;CIID;RPWPCR;91e647de-d96f-4b70-9557-d63ff4f3ccd8;;PS)(A;CIID;CCDCLCSWRPW\n"
+" PDTLOCRSDRCWDWO;;;S-1-5-21-2106703258-1007804629-1260019310-519)(A;CIID;LC;;;\n"
+" RU)(A;CIID;CCLCSWRPWPLOCRSDRCWDWO;;;BA)S:AI(OU;CIIOIDSA;WP;f30e3bbe-9ff0-11d1\n"
 " -b603-0000f80367c1;bf967aa5-0de6-11d0-a285-00aa003049e2;WD)(OU;CIIOIDSA;WP;f3\n"
 " 0e3bbf-9ff0-11d1-b603-0000f80367c1;bf967aa5-0de6-11d0-a285-00aa003049e2;WD)\n"
 "name: dda1d01d-4bd7-4c49-a184-46f9241b560e\n"
@@ -770,10 +770,10 @@ static bool torture_ldb_dn_extended(struct torture_context *torture)
 		       "Should find an GUID on extended DN");
 	
 	torture_assert_data_blob_equal(torture, *ldb_dn_get_extended_component(dn, "SID"), sid_blob, 
-				       "Extended DN SID incorect");
+				       "Extended DN SID incorrect");
 
 	torture_assert_data_blob_equal(torture, *ldb_dn_get_extended_component(dn, "GUID"), guid_blob, 
-				       "Extended DN GUID incorect");
+				       "Extended DN GUID incorrect");
 
 	torture_assert_str_equal(torture, ldb_dn_get_linearized(dn), dn_str, 
 				 "linearized DN incorrect");
@@ -818,10 +818,10 @@ static bool torture_ldb_dn_extended(struct torture_context *torture)
 		       "Failed to set a SID on DN");
 
 	torture_assert_data_blob_equal(torture, *ldb_dn_get_extended_component(dn, "SID"), sid_blob, 
-				       "Extended DN SID incorect");
+				       "Extended DN SID incorrect");
 
 	torture_assert_data_blob_equal(torture, *ldb_dn_get_extended_component(dn, "GUID"), guid_blob, 
-				       "Extended DN GUID incorect");
+				       "Extended DN GUID incorrect");
 
 	torture_assert_str_equal(torture, ldb_dn_get_linearized(dn), "cn=users,dc=samba,dc=org", 
 				 "linearized DN incorrect");
@@ -859,7 +859,7 @@ static bool torture_ldb_dn_extended(struct torture_context *torture)
 		       "Should find an GUID on this DN");
 	
 	torture_assert_data_blob_equal(torture, *ldb_dn_get_extended_component(dn, "GUID"), guid_blob, 
-				       "Extended DN GUID incorect");
+				       "Extended DN GUID incorrect");
 
 	torture_assert_str_equal(torture, ldb_dn_get_linearized(dn), "", 
 				 "linearized DN incorrect");
@@ -894,7 +894,7 @@ static bool torture_ldb_dn_extended(struct torture_context *torture)
 		       "Should find an GUID on this DN");
 	
 	torture_assert_data_blob_equal(torture, *ldb_dn_get_extended_component(dn, "GUID"), guid_blob, 
-				       "Extended DN GUID incorect");
+				       "Extended DN GUID incorrect");
 
 	torture_assert_str_equal(torture, ldb_dn_get_linearized(dn), "", 
 				 "linearized DN incorrect");
@@ -919,7 +919,7 @@ static bool torture_ldb_dn_extended(struct torture_context *torture)
 		       "Should find an SID on this DN");
 	
 	torture_assert_data_blob_equal(torture, *ldb_dn_get_extended_component(dn, "SID"), sid_blob, 
-				       "Extended DN SID incorect");
+				       "Extended DN SID incorrect");
 
 	torture_assert_str_equal(torture, ldb_dn_get_linearized(dn), "", 
 				 "linearized DN incorrect");
@@ -954,7 +954,7 @@ static bool torture_ldb_dn_extended(struct torture_context *torture)
 		       "Should find an SID on this DN");
 	
 	torture_assert_data_blob_equal(torture, *ldb_dn_get_extended_component(dn, "SID"), sid_blob, 
-				       "Extended DN SID incorect");
+				       "Extended DN SID incorrect");
 
 	torture_assert_str_equal(torture, ldb_dn_get_linearized(dn), "", 
 				 "linearized DN incorrect");
