@@ -78,8 +78,6 @@ static bool test_close_no_attrib(struct torture_context *tctx,
 				      ret, done, "Unexpected write time\n");
 	torture_assert_u64_equal_goto(tctx, c.out.change_time, NTTIME_OMIT,
 				      ret, done, "Unexpected change time\n");
-	torture_assert_u64_equal_goto(tctx, c.out.alloc_size, 0,
-				      ret, done, "Unexpected allocation size\n");
 	torture_assert_u64_equal_goto(tctx, c.out.size, 0,
 				      ret, done, "Unexpected size\n");
 	torture_assert_u64_equal_goto(tctx, c.out.file_attr, 0,
@@ -1066,8 +1064,6 @@ static bool test_delayed_2write(struct torture_context *tctx,
 	NTTIME create_time;
 	NTTIME write_time;
 	NTTIME write_time2;
-	struct timespec now;
-	NTTIME send_close_time;
 	NTTIME close_time;
 	NTSTATUS status;
 	bool ret = true;
@@ -1143,8 +1139,7 @@ static bool test_delayed_2write(struct torture_context *tctx,
 	torture_comment(tctx, "Close file-handle 1\n");
 	sleep(2);
 
-	now = timespec_current();
-	send_close_time = full_timespec_to_nt_time(&now);
+	torture_comment(tctx, "Check writetime has been updated\n");
 
 	c = (struct smb2_close) {
 		.in.file.handle = h1,
@@ -1157,7 +1152,7 @@ static bool test_delayed_2write(struct torture_context *tctx,
 	ZERO_STRUCT(h1);
 	close_time = c.out.write_time;
 
-	if (!(close_time > send_close_time)) {
+	if (!(close_time > write_time)) {
 		ret = false;
 		torture_fail_goto(tctx, done,
 				  "Write-time not updated (wrong!)\n");
@@ -1190,7 +1185,7 @@ struct torture_suite *torture_smb2_timestamps_init(TALLOC_CTX *ctx)
 	torture_suite_add_1smb2_test(suite, "freeze-thaw", test_freeze_thaw);
 
 	/*
-	 * Testing of delayed write-time udpates
+	 * Testing of delayed write-time updates
 	 */
 	torture_suite_add_1smb2_test(suite, "delayed-write-vs-seteof", test_delayed_write_vs_seteof);
 	torture_suite_add_1smb2_test(suite, "delayed-write-vs-flush", test_delayed_write_vs_flush);
